#include "canvas/Sun.hpp"

#include "Math.hpp"

#include <raylib.h>
#include <rlgl.h>

const char* sun_shader = "#version 330\n\
in vec2 fragTexCoord;\
in vec4 fragColor;\
out vec4 finalColor;\
uniform vec3 color;\
uniform float sun_radius;\
uniform float start_transperency;\
vec2 offset = vec2(1.0f, 1.0f);\
float sun_end = 1.0f;\
void main()\
{\
    offset.x -= fragTexCoord.x * 2;\
    offset.y -= fragTexCoord.y * 2;\
    float radius = length(offset);\
    if (radius < sun_radius) {\
        finalColor = vec4(color, 1.0f);\
    }\
    else if (radius < sun_end) {\
        float gradient = radius;\
        gradient -= sun_radius;\
        gradient = gradient / (sun_end - sun_radius) * start_transperency;\
        finalColor = vec4(color, start_transperency - gradient);\
    }\
    else {\
        finalColor = vec4(color, 0.0f);\
    }\
}";

void Sun::init()
{
  shader = LoadShaderFromMemory(0, sun_shader);
  target = LoadRenderTexture(sizeTexute, sizeTexute);

  sun_radius_loc = GetShaderLocation(shader, "sun_radius");
  SetShaderValue(shader, sun_radius_loc, &sun_radius, SHADER_UNIFORM_FLOAT);

  start_transperency_loc = GetShaderLocation(shader, "start_transperency");
  SetShaderValue(shader, start_transperency_loc, &start_transperency, SHADER_UNIFORM_FLOAT);

  colorLoc = GetShaderLocation(shader, "color");
  SetShaderValue(shader, colorLoc, c, SHADER_UNIFORM_VEC3);

  // resitev da ne postane tekstura okoli sonca transparentna in da se ne vidi nasledna slika 
  // https://github.com/raysan5/raylib/issues/3820
  rlSetBlendFactorsSeparate(RL_SRC_ALPHA, RL_ONE_MINUS_SRC_ALPHA, RL_ONE, RL_ONE, RL_FUNC_ADD, RL_MAX);
}

void Sun::deinit()
{
  UnloadShader(shader);
  UnloadRenderTexture(target);
}

void Sun::draw(float x, float y, float size)
{
  Rectangle dest = { x-size, y - size, size * 2, size * 2};
  // zgorni komentar da se mesanje barv oklopi pravilno
  BeginBlendMode(RL_BLEND_CUSTOM_SEPARATE);
    BeginShaderMode(shader);
    drawTexureWithRotation(target, dest, 0.0f);
    EndShaderMode();
  EndBlendMode();
}
